// MP3Play.js

(function(ext) {
    var device = null;
    var _rxBuf = [];

	var plays = {
        "play": 13,
		"pause": 14,
		"stop": 16,
		"next": 1,
		"previous": 2
	};
	
	var folderNames = {
        "001": 1,
		"002": 2,
		"003": 3,
		"004": 4,
		"005": 5,
		"006": 6,
		"007": 7,
		"008": 8,
		"009": 9,
		"010": 10
	};
	var fileNames = {
        "001": 1,
		"002": 2,
		"003": 3,
		"004": 4,
		"005": 5,
		"006": 6,
		"007": 7,
		"008": 8,
		"009": 9,
		"010": 10
	};
	
	var folders = {
		"nextfolder": 1,
		"previousfolder": 0
	};
		
	var vols = {
		"vol+": 4,
        "vol-": 5
	};
			
	var volumes = {
		"0": 0,
		"5": 5,
		"10": 10,
		"15": 15,
		"20": 20,
		"25": 25,
		"30": 30
	};
				
	var playmodes = {
		"all loop": 0,
		"folder loop": 1,
		"single loop": 2,
		"random play": 3,
		"stop loop": 4
	};
					
	var eqs = {
		"Normal": 0,
		"Pop": 1,
		"Rock": 2,
		"Jazz": 3,
		"Classic": 4,
		"Bass": 5
	};
	
	var startTimer = 0;
	var versionIndex = 0xFA;
    ext.resetAll = function(){
    	device.send([0xff, 0x55, 2, 0, 4]);
    };
	
	ext.MP3PlayCommand = function(command){
		if(typeof command=="string"){
			command = plays[command];
		}
		runPackage(105,1,command);
	};
	
	ext.MP3PlayFolderCommand = function(folder,file){
		if(typeof folder=="string"){
			folder = folderNames[folder];
		}
		if(typeof file=="string"){
			file = fileNames[file];
		}
		runPackage(105,2,folder,file);
	};
	
	ext.MP3SwitchFolderCommand = function(command){
		if(typeof command=="string"){
			command = folders[command];
		}
		runPackage(105,3,0x0F,command);
	};
	
	ext.MP3VolumeCommand = function(command){
		if(typeof command=="string"){
			command = vols[command];
		}		runPackage(105,4,command);
	};
	
	ext.MP3VolumeSetCommand = function(vol){
		if(typeof vol=="string"){
			vol = volumes[vol];
		}
		runPackage(105,5,0x06,vol);
	};
	
	ext.MP3ModeCommand = function(mode){
		if(typeof mode=="string"){
			mode = playmodes[mode];
		}
		runPackage(105,6,0x11,mode);
	};
	
	ext.MP3EQCommand = function(eq){
		if(typeof eq=="string"){
			eq = eqs[eq];
		}
		runPackage(105,7,0x07,eq);
	};
	
	function sendPackage(argList, type){
		var bytes = [0xff, 0x55, 0, 0, type];
		for(var i=0;i<argList.length;++i){
			var val = argList[i];
			if(val.constructor == "[class Array]"){
				bytes = bytes.concat(val);
			}else{
				bytes.push(val);
			}
		}
		bytes[2] = bytes.length - 3;
		device.send(bytes);
	}
	
	function runPackage(){
		sendPackage(arguments, 2);
	}
	function getPackage(){
		var nextID = arguments[0];
		Array.prototype.shift.call(arguments);
		sendPackage(arguments, 1);
	}

    var inputArray = [];
	var _isParseStart = false;
	var _isParseStartIndex = 0;
    function processData(bytes) {
		var len = bytes.length;
		if(_rxBuf.length>30){
			_rxBuf = [];
		}
		for(var index=0;index<bytes.length;index++){
			var c = bytes[index];
			_rxBuf.push(c);
			if(_rxBuf.length>=2){
				if(_rxBuf[_rxBuf.length-1]==0x55 && _rxBuf[_rxBuf.length-2]==0xff){
					_isParseStart = true;
					_isParseStartIndex = _rxBuf.length-2;
				}
				if(_rxBuf[_rxBuf.length-1]==0xa && _rxBuf[_rxBuf.length-2]==0xd&&_isParseStart){
					_isParseStart = false;
					
					var position = _isParseStartIndex+2;
					var extId = _rxBuf[position];
					position++;
					var type = _rxBuf[position];
					position++;
					//1 byte 2 float 3 short 4 len+string 5 double
					var value;
					switch(type){
						case 1:{
							value = _rxBuf[position];
							position++;
						}
							break;
						case 2:{
							value = readFloat(_rxBuf,position);
							position+=4;
							if(value<-255||value>1023){
								value = 0;
							}
						}
							break;
						case 3:{
							value = readInt(_rxBuf,position,2);
							position+=2;
						}
							break;
						case 4:{
							var l = _rxBuf[position];
							position++;
							value = readString(_rxBuf,position,l);
						}
							break;
						case 5:{
							value = readDouble(_rxBuf,position);
							position+=4;
						}
							break;
						case 6:
							value = readInt(_rxBuf,position,4);
							position+=4;
							break;
					}
					if(type<=6){
						responseValue(extId,value);
					}else{
						responseValue();
					}
					_rxBuf = [];
				}
			} 
		}
    }
	function readFloat(arr,position){
		var f= [arr[position],arr[position+1],arr[position+2],arr[position+3]];
		return parseFloat(f);
	}
	function readInt(arr,position,count){
		var result = 0;
		for(var i=0; i<count; ++i){
			result |= arr[position+i] << (i << 3);
		}
		return result;
	}
	function readDouble(arr,position){
		return readFloat(arr,position);
	}
	function readString(arr,position,len){
		var value = "";
		for(var ii=0;ii<len;ii++){
			value += String.fromCharCode(_rxBuf[ii+position]);
		}
		return value;
	}
    function appendBuffer( buffer1, buffer2 ) {
        return buffer1.concat( buffer2 );
    }

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);

        if (!device) {
            tryNextDevice();
        }
    }

    function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }

    var watchdog = null;
    function deviceOpened(dev) {
        if (!dev) {
            // Opening the port failed.
            tryNextDevice();
            return;
        }
        device.set_receive_handler('MP3Play',processData);
    };

    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    ext._getStatus = function() {
        if(!device) return {status: 1, msg: 'MP3Play disconnected'};
        if(watchdog) return {status: 1, msg: 'Probing for MP3Play'};
        return {status: 2, msg: 'MP3Play connected'};
    }

    var descriptor = {};
	ScratchExtensions.register('MP3Play', descriptor, ext, {type: 'serial'});
})({});
